StaticPopupDialogs["TRIANGULATION_NOCUT"] = {
	timeout=0,
	hideOnEscape=false,
	whileDead=false,
	showAlert=false,
	text="The survey data you just saved cannot be triangulated with the survey data saved for zone %s.\nWhich survey data do you want to keep?",
	button1="New data",
	button2="Old data",
	OnAccept=function(self, data, data2)
		Triangulation.lines[data] = data2
	end,
}

local function round(num, places)
	return tonumber(string.format("%."..(places or 0).."f",num))
end

local db
local addon = CreateFrame("Frame", "Triangulation")
addon:SetScript("OnEvent", function(self, event, ...)
	local arg = {...}
	if event == "ADDON_LOADED" and arg[1] == "Triangulation" then
		Triangulation_DB = Triangulation_DB or {}
		db = Triangulation_DB
		self.coordFrame:SetPoint("BOTTOMLEFT", db.offsetX or 300, db.offsetY or 300)
		self:UnregisterEvent("ADDON_LOADED")
		return
	end
	if event == "UNIT_SPELLCAST_SUCCEEDED" and arg[2] == self:GetCachedValue("SEARCH_ARTIFACT_SPELL") then
		self.coordFrame:Hide()
	end
end)
addon:RegisterEvent("ADDON_LOADED")

addon.lines={}
addon.points={}
addon.mapsizes={}
function addon:GetCurrentMapName()
	SetMapToCurrentZone()
	return (select((GetCurrentMapZone()),GetMapZones(GetCurrentMapContinent())))
end
function addon:GetCurrentMapBonds()
	local name = self:GetCurrentMapName()
	if self.mapsizes[name] then
		return unpack(self.mapsizes[name])
	else
		local x, y = select(2, GetMapInfo())
		self.mapsizes[name] = {x,y}
		return x,y
	end
end
function addon:GetMapCoords(x,y)
	local mX, mY = self:GetCurrentMapBonds()
	return x*mX, y*mY
end

local cachevalues = {
	["SEARCH_ARTIFACT_SPELL"] = function() return (GetSpellInfo(73979)) end,
}
addon.cache = {}
function addon:GetCachedValue(value)
	local v = addon.cache[value]
	if not v then
		v = cachevalues[value]()
		addon.cache[value] = v
	end
	return v
end

function addon:GetSlope()
	local facing = GetPlayerFacing()
	local gX = (facing > math.pi) and 1 or -1
	local yM = math.floor(facing/(math.pi/2))
	local gY = ((yM == 0) or (yM == 3)) and 1 or -1
	return tan(((((facing/(math.pi*2))*4)%2)-1)*90), gX, gY
end

function addon:GetCurrentLine()
	SetMapToCurrentZone()
	local px, py = self:GetMapCoords(GetPlayerMapPosition("player"))
	-- make bottom zero and top 1
	py=1-py
	local k, gX, gY = self:GetSlope()
	local d = py-(k*px)
	return k, d, px, py, gX, gY
end

function addon:Cut(k1, d1, k2, d2, p1x, p1y, p2x, p2y, growX1, growY1, growX2, growY2)
	if k1==k2 then return false end --lines have the same slope
	local s_x = (d2-d1)/(k1-k2)
	if (s_x-p1x)/growX1 < 0 then return false end
	if (s_x-p2x)/growX2 < 0 then return false end
	local s_y=k1*s_x+d1
	if (s_y-p1y)/growY1 < 0 then return false end
	if (s_y-p2y)/growY2 < 0 then return false end
	return s_x, s_y
end

function addon:CaptureSurvey()
	local curK, curD, curpX, curpY, curgX, curgY = self:GetCurrentLine()
	local zone = self:GetCurrentMapName()
	local prevL = self.lines[zone]
	if prevL then
		local cutX, cutY = self:Cut(curK, curD, prevL.k, prevL.d, curpX, curpY, prevL.pX, prevL.pY, curgX, curgY, prevL.gX, prevL.gY)
		cutY = 1-(cutY or 0) -- revert back to default (bottom equals 1, top equals zero) coordinating
		local maxX, maxY = self:GetCurrentMapBonds()
		if cutX and not ((cutX>maxX or cutX<0) or (cutY>maxY or cutY<0)) then
			print("Cut point found: "..cutX, cutY)
			self.lines[zone] = nil
			self.points[zone] = {x=cutX, y=cutY}
			self.coordFrame:Set(cutX,cutY)
		else
			local spd = StaticPopup_Show("TRIANGULATION_NOCUT", zone)
			spd.data = zone
			spd.data2 = {k=curK, d=curD, gX=curgX, gY=curgY, pX=curpX, pY=curpY}
		end
	else
		self.lines[zone]={k=curK,d=curD, gX=curgX, gY=curgY, pX=curpX, pY=curpY}
		print("|cffffd100Triangulation:|r Survey position and angle saved successfully")
	end
end

addon.coordFrame = CreateFrame("Frame","Triangulation_Coords",UIParent)
local cFrame = addon.coordFrame
cFrame:SetSize(200, 175)
cFrame:Hide()
cFrame:SetBackdrop({
	bgFile="Interface\\DialogFrame\\UI-DialogBox-Background",
	edgeFile="Interface\\DialogFrame\\UI-DialogBox-Border",
	tileSize=32,
	tile=true,
	insets={
		top=12,
		bottom=12,
		right=12,
		left=12,
	},
})
cFrame.hide = CreateFrame("Button", nil, cFrame, "UIPanelCloseButton")
cFrame.hide:SetSize(32,32)
cFrame.hide:SetPoint("TOPRIGHT")
cFrame:SetScript("OnUpdate", function(self)
	if WorldMapFrame:IsShown() then return end
	SetMapToCurrentZone()
	local x, y = addon:GetMapCoords(GetPlayerMapPosition("player"))
	local dX, dY = (self.tarX-x), 1-(self.tarY-y)
	local angle = atan(dY/dX)
	local tFold = "Interface\\AddOns\\Triangulation\\tex\\"
	if db.relativeArrow then
		local aAngle = (dX>0 and 270 or 90)+angle
		local pAngle = deg(GetPlayerFacing())
		local angleD = (aAngle-pAngle)%360
		self.tex:SetTexture(round(angle).."\n"..round(aAngle).."\n"..round(pAngle).."\n"..round(angleD))
	else
		if (angle > (-22.5)) and (angle < 22.5) then
			-- either right or left
			if dX > 0 then
				
				self.tex:SetTexture(tFold.."arr-right")
			else
				self.tex:SetTexture(tFold.."arr-left")
			end
		elseif (angle > 22.5) and (angle < 67.5) then
			--topright or bottomleft
			if dX > 0 then
				self.tex:SetTexture(tFold.."arr-topright")
			else
				self.tex:SetTexture(tFold.."arr-bottomleft")
			end
		elseif ((angle > 67.5) and (angle < 90)) or ((angle > (-90)) and (angle < (-67.5))) then
			-- top or bottom
			if dY > 0 then
				self.tex:SetTexture(tFold.."arr-top")
			else
				self.tex:SetTexture(tFold.."arr-bottom")
			end
		else
			--topleft or bottomright
			if dX < 0 then
				self.tex:SetTexture(tFold.."arr-topleft")
			else
				self.tex:SetTexture(tFold.."arr-bottomright")
			end
		end
	end
	self.text.text1:SetText(round(x, 2))
	self.text.text2:SetText(round(y, 2))
end)
for i=1, 2 do
	local f = CreateFrame("Frame", nil, cFrame)
	f:SetSize(170, 30)
	f.text1 = f:CreateFontString(nil, "ARTWORK")
	f.text1:SetFont("Fonts\\FRIZQT__.TTF", 14, "OUTLINE")
	local textcolor = i==1 and {1,1,1} or {1,0.82,0}
	f.text1:SetTextColor(unpack(textcolor))
	f.text1:SetPoint("TOPLEFT")
	f.text1:SetPoint("BOTTOMRIGHT", f, "BOTTOM")
	f.text2 = f:CreateFontString(nil, "ARTWORK")
	f.text2:SetFont("Fonts\\FRIZQT__.TTF", 14, "OUTLINE")
	f.text2:SetTextColor(unpack(textcolor))
	f.text2:SetPoint("TOPLEFT", f, "TOP")
	f.text2:SetPoint("BOTTOMRIGHT")
	if i == 1 then
		f:SetPoint("TOPLEFT", 15, -100)
		cFrame.text = f
	else
		f:SetPoint("TOPLEFT", cFrame.text, "BOTTOMLEFT", 0, -5)
		function cFrame:Set(x,y)
			self:Show()
			self.tarX = x
			self.tarY = y
			f.text1:SetText(round(x,2))
			f.text2:SetText(round(y,2))
		end
	end
end
cFrame.tex = cFrame:CreateTexture()
cFrame.tex:SetSize(70,70)
cFrame.tex:SetPoint("TOP", 0, -25)

SlashCmdList["TRIANGULATION"]=function() addon:CaptureSurvey() end
SLASH_TRIANGULATION1="/triangulation"
SLASH_TRIANGULATION2="/survey"